from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes, hmac


def hmac_with_sha1(password: bytes, string: bytes):
    hash_result = hmac.HMAC(
        password,
        algorithm=hashes.SHA1(),  # noqa: S303
        backend=default_backend(),
    )
    hash_result.update(string)
    return hash_result.finalize().hex().upper()



from datetime import datetime


def iso8601_without_underscore(date=None):
    if date:
        iso8601 = date.replace(microsecond=0).isoformat()
    else:
        iso8601 = datetime.now().replace(microsecond=0).isoformat()
    return iso8601.replace('-', '')



from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='python-tutuka-client-testing',
    version='1.3.0',
    author='Fondeadora',
    author_email='tech@fondeadora.com',
    description='Tutuka XML-RPC Client',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    url='https://github.com/Fondeadora/tutuka-client',
    keywords=['tutuka', 'client'],
    python_requires='>=3.6',
    install_requires=['requests>=2.24.0', 'cryptography>=2.9.2'],
)



from types import MappingProxyType

from tutuka_client import errors

SUCCESS_RESPONSE_CODES = MappingProxyType({
    'APPROVED_NO_ACTION': 0,
    'APPROVED': 1,
})

ERROR_RESPONSE_CODES_EXCEPTIONS = MappingProxyType({
    '-3': errors.DuplicateTransaction,
    '-4': errors.InvalidCardID,
    '-5': errors.OperationNotAllowed,
    '-6': errors.OperationNotSupported,
    '-7': errors.TransactionTimeout,
    '-8': errors.AuthenticationFailed,
    '-9': errors.OperationDeclined,
    '-17': errors.InsufficientFunds,
    '-18': errors.WithdrawalsLimitExceeded,
    '-19': errors.InvalidAmount,
    '-24': errors.SecurityViolation,
    '-25': errors.IncorrectPin,
    '-26': errors.PinTriesExceeded,
    '-27': errors.InvalidPinBlock,
    '-28': errors.PinLengthError,
    '-34': errors.CardAlreadyActive,
    '-35': errors.CardNotActive,
    '-36': errors.ExpiredCard,
    '-37': errors.SuspectFraud,
    '-38': errors.LostCard,
    '-39': errors.StolenCard,
    '-100': errors.MonthlyLoadAmountExceeded,
    '-101': errors.BlockedAccount,
    '-102': errors.InvalidClient,
    '-103': errors.InvalidAccount,
    '-244': errors.InvalidLastName,
    '-246': errors.InvalidFirstName,
    '-247': errors.InvalidID,
    '-248': errors.InvalidMSISDN,
    '-778': errors.ReferenceWithoutLinkedCard,
    '-779': errors.ReferenceWithDuplicatedLinkedCard,
})



class TutukaException(Exception):
    def __init__(self, message='', code=None):
        self.code = code
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return '<{class_name} {code}: {message}>'.format(
            class_name=self.__class__.__name__,
            code=self.code,
            message=self.message,
        )


class DuplicateTransaction(TutukaException):
    """Duplicate Transaction ID."""


class InvalidCardID(TutukaException):
    """Invalid Card Identifier."""


class OperationNotAllowed(TutukaException):
    """Operation not Allowed."""


class OperationNotSupported(TutukaException):
    """Operation not Supported."""


class TransactionTimeout(TutukaException):
    """Transaction Timeout."""


class AuthenticationFailed(TutukaException):
    """Authentication Failed."""


class OperationDeclined(TutukaException):
    """Do not honor (general decline, no specific reason given)."""


class CardAlreadyActive(TutukaException):
    """Card already active."""


class CardNotActive(TutukaException):
    """Card not active."""


class ExpiredCard(TutukaException):
    """Expired card."""


class LostCard(TutukaException):
    """Lost card."""


class StolenCard(TutukaException):
    """Stolen card."""


class InvalidLastName(TutukaException):
    """Invalid last name."""


class InvalidFirstName(TutukaException):
    """Invalid first name."""


class InvalidID(TutukaException):
    """Invalid ID number."""


class InvalidMSISDN(TutukaException):
    """Invalid MSISDN."""


class ReferenceWithoutLinkedCard(TutukaException):
    """Reference has no linked cards."""


class ReferenceWithDuplicatedLinkedCard(TutukaException):
    """Card already linked to a different reference."""


class InsufficientFunds(TutukaException):
    """No enough funds for operation."""


class WithdrawalsLimitExceeded(TutukaException):
    """Exceeds number of withdrawals."""


class InvalidAmount(TutukaException):
    """Amount is not valid."""


class SecurityViolation(TutukaException):
    """Something went wrong during authorization."""


class IncorrectPin(TutukaException):
    """Not valid PIN."""


class PinTriesExceeded(TutukaException):
    """Exceeds number of pin tries."""


class InvalidPinBlock(TutukaException):
    """Invalid PIN block."""


class PinLengthError(TutukaException):
    """Length of PIN is not valid."""


class SuspectFraud(TutukaException):
    """Unusual operation detected."""


class MonthlyLoadAmountExceeded(TutukaException):
    """Exceeds allowed amount."""


class InvalidClient(TutukaException):
    """Invalid Client."""


class InvalidAccount(TutukaException):
    """Invalid Account."""


class BlockedAccount(TutukaException):
    """Account blocked."""



from types import MappingProxyType

CARD_STOP_REASONS = MappingProxyType({
    'CARD_LOST': 1,
    'CARD_STOLEN': 2,
    'PENDING_QUERY': 3,
    'CARD_CONSOLIDATION': 4,
    'CARD_INACTIVE': 5,
    'PIN_TRIES_EXCEEDED': 6,
    'SUSPECTED_FRAUD': 7,
    'CARD_REPLACED': 8,
})



from datetime import datetime

from tutuka_client.base import BaseClient
from tutuka_client.errors import TutukaException
from tutuka_client.stop_reasons import CARD_STOP_REASONS
from tutuka_client.utils import date_time_normalize


class LocalApiClient(BaseClient):

    def create_linked_card(
        self,
        reference_id: str,
        first_name: str,
        last_name: str,
        id_or_passport: str,
        expiry_date: datetime,
        transaction_id: str,
        cell_phone_number: str = '',
    ):
        return self.execute(
            method_name='CreateLinkedCard',
            arguments=[
                self.terminal_id,
                reference_id,
                first_name,
                last_name,
                id_or_passport,
                cell_phone_number,
                expiry_date,
                transaction_id,
            ],
        )

    def link_card(
        self,
        reference_id: str,
        card_identifier: str,
        first_name: str,
        last_name: str,
        id_or_passport: str,
        transaction_id: str,
        cell_phone_number: str = '',
    ):
        return self.execute(
            method_name='LinkCard',
            arguments=[
                self.terminal_id,
                reference_id,
                card_identifier,
                first_name,
                last_name,
                id_or_passport,
                cell_phone_number,
                transaction_id,
            ],
        )

    def order_card(
        self,
        title: str,
        last_name: str,
        transaction_id: str,
        initials: str = '',
        address1: str = '',
        address2: str = '',
        address3: str = '',
        address4: str = '',
        address5: str = '',
        additional_data: str = '',
    ):
        return self.execute(
            method_name='OrderCard',
            arguments=[
                self.terminal_id,
                title,
                initials,
                last_name,
                address1,
                address2,
                address3,
                address4,
                address5,
                additional_data,
                transaction_id,
            ],
        )

    def order_card_with_pin_block(  # noqa: WPS211
        self,
        title: str,
        last_name: str,
        pin_block: str,
        transaction_id: str,
        initials: str = '',
        address1: str = '',
        address2: str = '',
        address3: str = '',
        address4: str = '',
        address5: str = '',
        additional_data: str = '',
    ):
        return self.execute(
            method_name='OrderCardWithPinBlock',
            arguments=[
                self.terminal_id,
                title,
                initials,
                last_name,
                address1,
                address2,
                address3,
                address4,
                address5,
                additional_data,
                pin_block,
                transaction_id,
            ],
        )

    def activate_card(self, card_identifier: str, transaction_id: str):
        return self.execute(
            method_name='ActivateCard',
            arguments=[
                self.terminal_id,
                card_identifier,
                transaction_id,
            ],
        )

    def get_active_linked_cards(self, reference_id: str, transaction_id: str):
        active_linked_cards = self.execute(
            method_name='GetActiveLinkedCards',
            arguments=[
                self.terminal_id,
                reference_id,
                transaction_id,
            ],
        )
        active_linked_cards['cards'] = [
            {
                key: date_time_normalize(value)
                for (key, value) in card.items()  # noqa: WPS110
            } for card in active_linked_cards['cards']
        ]
        return active_linked_cards

    def get_linked_cards(self, reference_id: str, transaction_id: str):
        linked_cards = self.execute(
            method_name='GetLinkedCards',
            arguments=[
                self.terminal_id,
                reference_id,
                transaction_id,
            ],
        )
        linked_cards['cards'] = [
            {
                key: date_time_normalize(value)
                for (key, value) in card.items()  # noqa: WPS110
            } for card in linked_cards['cards']
        ]
        return linked_cards

    def change_pin(
        self,
        reference_id: str,
        card_identifier: str,
        new_pin: str,
        transaction_id: str,
    ):
        return self.execute(
            method_name='ChangePin',
            arguments=[
                self.terminal_id,
                reference_id,
                card_identifier,
                new_pin,
                transaction_id,
            ],
        )

    def reset_pin(self, reference_id: str, card_identifier: str, transaction_id: str):
        return self.execute(
            method_name='ResetPin',
            arguments=[
                self.terminal_id,
                reference_id,
                card_identifier,
                transaction_id,
            ],
        )

    def update_bearer(
        self,
        reference_id: str,
        card_identifier: str,
        first_name: str,
        last_name: str,
        id_or_passport: str,
        cell_phone_number: str,
        transaction_id: str,
    ):
        return self.execute(
            method_name='UpdateBearer',
            arguments=[
                self.terminal_id,
                reference_id,
                card_identifier,
                first_name,
                last_name,
                id_or_passport,
                cell_phone_number,
                transaction_id,
            ],
        )

    def transfer_link(
        self,
        reference_id: str,
        old_card_identifier: str,
        new_card_identifier: str,
        transaction_id: str,
    ):
        return self.execute(
            method_name='TransferLink',
            arguments=[
                self.terminal_id,
                reference_id,
                old_card_identifier,
                new_card_identifier,
                transaction_id,
            ],
        )

    def stop_card(
        self,
        reference_id: str,
        card_identifier: str,
        reason: str,
        note: str,
        transaction_id: str,
    ):
        if reason not in CARD_STOP_REASONS:
            raise TutukaException(
                'invalid card stop reason: {reason}'.format(reason=reason),
            )

        return self.execute(
            method_name='StopCard',
            arguments=[
                self.terminal_id,
                reference_id,
                card_identifier,
                CARD_STOP_REASONS[reason],
                note,
                transaction_id,
            ],
        )

    def unstop_card(
        self,
        reference_id: str,
        card_identifier: str,
        note: str,
        transaction_id: str,
    ):
        return self.execute(
            method_name='UnstopCard',
            arguments=[
                self.terminal_id,
                reference_id,
                card_identifier,
                note,
                transaction_id,
            ],
        )

    def status(self, reference_id: str, card_identifier: str, transaction_id: str):
        return self.execute(
            method_name='Status',
            arguments=[
                self.terminal_id,
                reference_id,
                card_identifier,
                transaction_id,
            ],
        )

    def set3d_secure_code(
        self,
        reference_id: str,
        card_identifier: str,
        code: str,
        transaction_id: str,
    ):
        return self.execute(
            method_name='Set3dSecureCode',
            arguments=[
                self.terminal_id,
                reference_id,
                card_identifier,
                code,
                transaction_id,
            ],
        )

    def update_cvv(
        self,
        reference_id: str,
        card_identifier: str,
        transaction_id: str,
    ):
        return self.execute(
            method_name='UpdateCVV',
            arguments=[
                self.terminal_id,
                reference_id,
                card_identifier,
                transaction_id,
            ],
        )

    def retire_card(
        self,
        reference_id: str,
        card_identifier: str,
        transaction_id: str,
    ):
        return self.execute(
            method_name='RetireCard',
            arguments=[
                self.terminal_id,
                reference_id,
                card_identifier,
                transaction_id,
            ],
        )

    def administrative_message(
        self,
        reference_id: str,
        message_name: str,
        transaction_id: str,
        message_data: str = '',
    ):
        return self.execute(
            method_name='AdministrativeMessage',
            arguments=[
                self.terminal_id,
                reference_id,
                message_name,
                message_data,
                transaction_id,
            ],
        )
